//
// Proof-of-concept: IO on P15
// 
// Target is AT91R40008 on NavMe-Board
// Toolchain is GNUARM 4.1.0
//
// May 21, 2006 v0.1 Stefan
//
// UART0 is communication between AT91R40008 and world (DSUB26, 4800-8n1-no handshake)
// UART1 is communication between Tyco module and AT91R40008 (on PCB)
// 

#include "at91x40.h"		// unmodified from WINARM 4.1.0 package
#include "at91x40_bit.h"	// new

// Switch on/off changes to former bidirectional UART example without IO on P15
#define IOMOD
// see main()
#define IS_INPUT (!(PIO_OSR & (1<<P15_RXD0)))

/*
***************************************************************************************
** UART0 is communication between AT91R40008 and world (DSUB26, 4800-8n1-no handshake)
***************************************************************************************
*/
static void uart0Init(void)
{
#ifdef IOMOD
	// RXD0 is not used, but P15 as IO pin
	// default after RESET: P14 is INPUT

	// Define TXD only as peripheral
	PIO_PDR |=  (1<<P14_TXD0);
	PIO_PER &= ~(1<<P14_TXD0);
#else
	// Define RXD and TXD as peripheral
	PIO_PDR |=   (1<<P14_TXD0) | (1<<P15_RXD0);
	PIO_PER &= ~((1<<P14_TXD0) | (1<<P15_RXD0));
#endif /* ! IOMOD */

	// First, enable the clock
	// US0
	PS_PCER |= (1<<2);

	// Reset receiver and transmitter
	//       COM0->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX;

	// Reset Status Bits (PARE, FRAME, OVRE and RXBRK)
	//       COM0->US_CR = AT91C_US_RSTSTA;

	// Baudrate
	US0_BRGR = (66000000UL / 4800UL) / 16; 

	// async, 8 Bit, N, 1, normal mode, 
	US0_MR = 0 | (1<<CHRL1) | (1<<CHRL0) | (1<<PAR2);

#ifdef IOMOD
	// Stop Peripheral Data Transfer
	US0_TCR = 0;

	// TX enable
	US0_CR = 0 | (1<<TXEN);
#else
	// Stop Peripheral Data Transfer
	US0_RCR = 0;
	US0_TCR = 0;

	// RX, TX enable
	US0_CR = 0 | (1<<RXEN) | (1<<TXEN);
#endif /* ! IOMOD */
}

static void uart0Putch(unsigned char data)
{
	while(!(US0_CSR & ((1<<TXRDY)|(1<<TXEMPTY))))
		;
	US0_THR = (unsigned long) data;
}

static void uart0PutString(const char *s)
{
	while(*s)
		uart0Putch(*s++);
}

/*
// not used
static unsigned char uart0Getch(void)
{
	unsigned char data = '+';
  
	while(!(US0_CSR & (1<<RXRDY)))
		;
	data = (unsigned char) (US0_RHR & 0x000000FF);
	return data;
}
*/

/*
**********************************************************************
** UART1 is communication between Tyco module and AT91R40008 (on PCB)
**********************************************************************
*/ 
static void uart1Init(void)
{
	// Define RXD and TXD as peripheral
	PIO_PDR |=   (1<<P21_TXD1) | (1<<P22_RXD1);
	PIO_PER &= ~((1<<P21_TXD1) | (1<<P22_RXD1));

	// First, enable the clock
	// US1
	PS_PCER |= (1<<3);

	// Reset receiver and transmitter
	//       COM0->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX;

	// Reset Status Bits (PARE, FRAME, OVRE and RXBRK)
	//       COM0->US_CR = AT91C_US_RSTSTA;

	// Baudrate
	US1_BRGR = (66000000UL / 4800UL) / 16; 

	// async, 8 Bit, N, 1, normal mode, 
	US1_MR = 0 | (1<<CHRL1) | (1<<CHRL0) | (1<<PAR2);

	// Stop Peripheral Data Transfer
	US1_RCR = 0;
	US1_TCR = 0;

	// RX, TX enable
	US1_CR = 0 | (1<<RXEN) | (1<<TXEN);
}


static unsigned char uart1Getch(void)
{
	unsigned char data = '+';
	
	while(!(US1_CSR & (1<<RXRDY)))
		;
	data = (unsigned char) (US1_RHR & 0x000000FF);
	return data;
}

/*
********
** MAIN
********
*/ 
int main(void)
{
	// Set output P16 HIGH to ENABLE Tyco module
	PIO_PER  |= (1<<16);		// ENABLE PIO CONTROL PIN16
	PIO_OER  |= (1<<16);		// ENABLE PIO OUTPUT
	PIO_SODR |= (1<<16);		// SET PIO OUTPUT

	// Initialize UARTs
	uart0Init();
	uart1Init();

	// Send welcome message to world
	// Definition RAM_RUN and ROM-RUN see makefile
#ifdef RAM_RUN  
	uart0PutString("\fgcc-uart-polling (RAM_RUN)\n\r\n\r");
#endif 
#ifdef ROM_RUN
	uart0PutString("\fgcc-uart-polling (ROM_RUN)\n\r\n\r");
#endif 

	// work forever... get char from Tyco module and send char to world
	while(1) 
	{
		unsigned char ch = '#';
		static unsigned char line = 1;
		
		ch = uart1Getch();

#ifdef IOMOD
		// count lines for output example
		if ('\n' == ch) 
			line += 1;

		// LOW on input P15 ?
		// = Switch to GND closed
		if (IS_INPUT && !(PIO_PDSR & (1<<P15_RXD0))) 
		{
			ch = '%';	// do something
		}

		// every 10th line LED off 
		if (IS_INPUT && (10 == line))
		{
			// config P15 as output pin and output LOW
			PIO_PER  |= (1<<P15_RXD0);		// PIO CONTROL PIN15
			PIO_OER  |= (1<<P15_RXD0);		// PIO OUTPUT ENABLE
			PIO_CODR |= (1<<P15_RXD0);		// CLEAR PIO OUTPUT => LOW
		}
		
		// 3 lines after 10th line turn output off and input on again
		if (!IS_INPUT && (10+3 < line))
		{
			line = 1;

			// config P15 as input pin again
			PIO_PER  |= (1<<P15_RXD0);		// PIO CONTROL PIN15
			PIO_ODR  |= (1<<P15_RXD0);		// PIO OUTPUT DISABLE
		}
#endif /* IOMOD */		
				
		uart0Putch(ch);
	}

	return 0;	// never reached!
}

/* EOF */
